@@ -100,18 +100,244 @@ sub _rt_content_to_exception {
 }
 
 
+# This package provides functions from RT::Interface::REST, in case
+# that module is not installed.  This one is simply copied from rt 3.4.5.
+package RT::Client::REST::Aux;
+
+use strict;
+use warnings;
+use Exporter;
+
+use vars qw(@EXPORT @ISA);
+
+@ISA = qw(Exporter);
+@EXPORT = qw(expand_list form_parse form_compose vpush vsplit);
+
+my $field = '[a-zA-Z][a-zA-Z0-9_-]*';
+
+sub expand_list {
+    my ($list) = @_;
+    my ($elt, @elts, %elts);
+
+    foreach $elt (split /,/, $list) {
+        if ($elt =~ /^(\d+)-(\d+)$/) { push @elts, ($1..$2) }
+        else                         { push @elts, $elt }
+    }
+
+    @elts{@elts}=();
+    return sort {$a<=>$b} keys %elts;
+}
+
+# Returns a reference to an array of parsed forms.
+sub form_parse {
+    my $state = 0;
+    my @forms = ();
+    my @lines = split /\n/, $_[0];
+    my ($c, $o, $k, $e) = ("", [], {}, "");
+
+    LINE:
+    while (@lines) {
+        my $line = shift @lines;
+
+        next LINE if $line eq '';
+
+        if ($line eq '--') {
+            # We reached the end of one form. We'll ignore it if it was
+            # empty, and store it otherwise, errors and all.
+            if ($e || $c || @$o) {
+                push @forms, [ $c, $o, $k, $e ];
+                $c = ""; $o = []; $k = {}; $e = "";
+            }
+            $state = 0;
+        }
+        elsif ($state != -1) {
+            if ($state == 0 && $line =~ /^#/) {
+                # Read an optional block of comments (only) at the start
+                # of the form.
+                $state = 1;
+                $c = $line;
+                while (@lines && $lines[0] =~ /^#/) {
+                    $c .= "\n".shift @lines;
+                }
+                $c .= "\n";
+            }
+            elsif ($state <= 1 && $line =~ /^($field):(?:\s+(.*))?$/) {
+                # Read a field: value specification.
+                my $f  = $1;
+                my @v  = ($2 || ());
+
+                # Read continuation lines, if any.
+                while (@lines && ($lines[0] eq '' || $lines[0] =~ /^\s+/)) {
+                    push @v, shift @lines;
+                }
+                pop @v while (@v && $v[-1] eq '');
+
+                # Strip longest common leading indent from text.
+                my ($ws, $ls) = ("");
+                foreach $ls (map {/^(\s+)/} @v[1..$#v]) {
+                    $ws = $ls if (!$ws || length($ls) < length($ws));
+                }
+                s/^$ws// foreach @v;
+
+                push(@$o, $f) unless exists $k->{$f};
+                vpush($k, $f, join("\n", @v));
+
+                $state = 1;
+            }
+            elsif ($line !~ /^#/) {
+                # We've found a syntax error, so we'll reconstruct the
+                # form parsed thus far, and add an error marker. (>>)
+                $state = -1;
+                $e = form_compose([[ "", $o, $k, "" ]]);
+                $e.= $line =~ /^>>/ ? "$line\n" : ">> $line\n";
+            }
+        }
+        else {
+            # We saw a syntax error earlier, so we'll accumulate the
+            # contents of this form until the end.
+            $e .= "$line\n";
+        }
+    }
+    push(@forms, [ $c, $o, $k, $e ]) if ($e || $c || @$o);
+
+    my $l;
+    foreach $l (keys %$k) {
+        $k->{$l} = vsplit($k->{$l}) if (ref $k->{$l} eq 'ARRAY');
+    }
+
+    return \@forms;
+}
+
+# Returns text representing a set of forms.
+sub form_compose {
+    my ($forms) = @_;
+    my (@text, $form);
+
+    foreach $form (@$forms) {
+        my ($c, $o, $k, $e) = @$form;
+        my $text = "";
+
+        if ($c) {
+            $c =~ s/\n*$/\n/;
+            $text = "$c\n";
+        }
+        if ($e) {
+            $text .= $e;
+        }
+        elsif ($o) {
+            my (@lines, $key);
+
+            foreach $key (@$o) {
+                my ($line, $sp, $v);
+                my @values = (ref $k->{$key} eq 'ARRAY') ?
+                               @{ $k->{$key} } :
+                                  $k->{$key};
+
+                $sp = " "x(length("$key: "));
+                $sp = " "x4 if length($sp) > 16;
+
+                foreach $v (@values) {
+                    if ($v =~ /\n/) {
+                        $v =~ s/^/$sp/gm;
+                        $v =~ s/^$sp//;
+
+                        if ($line) {
+                            push @lines, "$line\n\n";
+                            $line = "";
+                        }
+                        elsif (@lines && $lines[-1] !~ /\n\n$/) {
+                            $lines[-1] .= "\n";
+                        }
+                        push @lines, "$key: $v\n\n";
+                    }
+                    elsif ($line &&
+                           length($line)+length($v)-rindex($line, "\n") >= 70)
+                    {
+                        $line .= ",\n$sp$v";
+                    }
+                    else {
+                        $line = $line ? "$line, $v" : "$key: $v";
+                    }
+                }
+
+                $line = "$key:" unless @values;
+                if ($line) {
+                    if ($line =~ /\n/) {
+                        if (@lines && $lines[-1] !~ /\n\n$/) {
+                            $lines[-1] .= "\n";
+                        }
+                        $line .= "\n";
+                    }
+                    push @lines, "$line\n";
+                }
+            }
+
+            $text .= join "", @lines;
+        }
+        else {
+            chomp $text;
+        }
+        push @text, $text;
+    }
+
+    return join "\n--\n\n", @text;
+}
+
+# Add a value to a (possibly multi-valued) hash key.
+sub vpush {
+    my ($hash, $key, $val) = @_;
+    my @val = ref $val eq 'ARRAY' ? @$val : $val;
+
+    if (exists $hash->{$key}) {
+        unless (ref $hash->{$key} eq 'ARRAY') {
+            my @v = $hash->{$key} ne '' ? $hash->{$key} : ();
+            $hash->{$key} = \@v;
+        }
+        push @{ $hash->{$key} }, @val;
+    }
+    else {
+        $hash->{$key} = $val;
+    }
+}
+
+# "Normalise" a hash key that's known to be multi-valued.
+sub vsplit {
+    my ($val) = @_;
+    my ($line, $word, @words);
+
+    foreach $line (map {split /\n/} (ref $val eq 'ARRAY') ? @$val : $val)
+    {
+        # XXX: This should become a real parser, à la Text::ParseWords.
+        $line =~ s/^\s+//;
+        $line =~ s/\s+$//;
+        push @words, split /\s*,\s*/, $line;
+    }
+
+    return \@words;
+}
+
 # Now for the actual package.
 package RT::Client::REST;
-use vars qw/$VERSION/;
-
-$VERSION = 0.05;
 
 use strict;
 use warnings;
 
-    # This exports the following methods:
-    # expand_list form_parse form_compose vpush vsplit
-use RT::Interface::REST;
+use vars qw/$VERSION/;
+$VERSION = 0.06;
+
+# If RT::Interface::REST, use it.  Otherwise, use local copy.
+eval { require RT::Interface::REST };
+
+# This exports the following methods:
+# expand_list form_parse form_compose vpush vsplit
+if ($@) {
+    for my $function (qw(expand_list form_parse form_compose vpush vsplit)) {
+        no strict 'refs';
+        *{$function} = \&{'RT::Client::REST::Aux::' . $function};
+    }
+} else {
+    eval "use RT::Interface::REST;";
+}
 
 use LWP;
 use HTTP::Cookies;
@@ -194,7 +420,13 @@ sub edit {
         );
     }
 
-    return;
+    if ($r->content =~ /^#[^\d]+(\d+) (?:created|updated)/) {
+        return $1;
+    } else {
+        RT::Client::REST::MalformedRTResponseException->throw(
+            message => "Cound not read ID of the modified object",
+        );
+    }
 }
 
 sub create { shift->edit(@_, objects => ['new']) }
@@ -509,7 +741,7 @@ to implement some RT interactions from my application, but did not feel that
 invoking a shell command is appropriate.  Thus, I took B<rt> tool, written
 by Abhijit Menon-Sen, and converted it to an object-oriented Perl module.
 
-As of this writing (version 0.05), B<RT::Client::REST> is missing a lot of
+As of this writing (version 0.06), B<RT::Client::REST> is missing a lot of
 things that B<rt> has.  It does not support attachments, CCs, BCCs, and
 probably other things.  B<RT::Client::REST> does not retrieve forms from
 RT server, which is either good or bad, depending how you look at it.  More
@@ -565,6 +797,8 @@ prescribed by the B<set> parameter.
 =item create (type => $type, set => \%params)
 
 Create a new object of type B<$type> and set initial parameters to B<%params>.
+Returns numeric ID of the new object.  If numeric ID cannot be parsed from
+the response, B<RT::Client::REST::MalformedRTResponseException> is thrown.
 
 =item comment (ticket_id => $id, message => "This is a comment")
 
@@ -700,10 +934,6 @@ Exception::Class
 
 =item
 
-RT::Interface::REST
-
-=item
-
 LWP
 
 =item
@@ -736,7 +966,7 @@ Implement /usr/bin/rt using this RT::Client::REST.
 
 =head1 VERSION
 
-This is version 0.05 of B<RT::Client::REST>.  B</usr/bin/rt> shipped with
+This is version 0.06 of B<RT::Client::REST>.  B</usr/bin/rt> shipped with
 RT 3.4.5 is version 0.02, so the logical version continuation is to have
 one higher.
 
